open Integer
open Primitive

(* [prim2label] maps primitive operations to the names under which
   they appear in the assembly code. These names must of course
   correspond to labels in the assembly program that is produced.
   This is ensured in [PrintASM] by appending standard definitions for
   these labels to the [ASM] program. *)

let prim2label = function
  | Write ->
      "write"
  | Writeln ->
      "writeln"
  | Readln ->
      "readln"
  | Alloc ->
      "alloc"

(* [proc2label p f] maps the procedure name [f] to a procedure entry
   label. *)

let proc2label (p : LIN.program) f =
  let proc = StringMap.find f p.LIN.defs in
  match proc.LIN.code with
  | LIN.ILabel entry :: _ ->
      entry
  | _ ->
      assert false

(* [locals proc] is the size of the [proc]'s local stack area. *)

let locals proc = failwith "Pas implment !"

(* [formals proc] is the size of the [proc]'s incoming stack area. *)

let formals proc = failwith "Pas implment !"

(* [translate_slot proc slot] translates [slot] into an offset off
   [$sp]. *)

let translate_slot proc slot = failwith "Pas implment !"

(* [adjust offset] generates an [ASM] instruction that adjusts the
   stack pointer [sp] by [offset]. *)

let adjust offset = failwith "Pas implment !"

(* [translate_instruction p proc i] translates the instruction [i]. *)

let translate_instruction
    (p : LIN.program)
    (proc : LIN.procedure)
    (instruction : LIN.instruction)
    : ASM.instruction
    =
  failwith "Pas implment !"

(* [cons i is] preprends instruction [i] in front of the instruction
   sequence [is], unless it is an [INop] instruction. *)

let cons i is =
  match i with
  | ASM.INop ->
      is
  | _ ->
      i :: is

(* [translate_procedure p f proc instructions] prepends a translation
   of procedure [proc] in front of the instruction sequence
   [instructions]. A couple of comments are inserted at the beginning
   and end of the procedure to help identify it. *)

let translate_procedure
    (p : LIN.program)
    (name : string)
    (proc : LIN.procedure)
    (instructions : ASM.instructions)
    : ASM.instructions
    =
  ASM.IComment (true, "begin " ^ name) ::
  List.fold_right (fun instruction instructions ->
    cons (translate_instruction p proc instruction) instructions
  ) proc.LIN.code (
  ASM.IComment (false, "end " ^ name) ::
  instructions
  )

(* [translate_program p] translates the program [p]. *)

let translate_program (p : LIN.program) : ASM.program = {
  ASM.globals = p.LIN.globals;
  ASM.entry   = Label.print (proc2label p "_main");
  ASM.code    = StringMap.fold (translate_procedure p) p.LIN.defs []
}

